@@ -164,7 +164,7 @@ async fn test_cancel() -> PyResult<()> {
             assert!(e.value(py).is_instance(
                 py.import("asyncio")?
                     .getattr("CancelledError")?
-                    .downcast::<PyType>()
+                    .cast::<PyType>()
                     .unwrap()
             )?);
             Ok(())
@@ -216,7 +216,7 @@ fn test_local_cancel(event_loop: Py<PyAny>) -> PyResult<()> {
                     assert!(e.value(py).is_instance(
                         py.import("asyncio")?
                             .getattr("CancelledError")?
-                            .downcast::<PyType>()
+                            .cast::<PyType>()
                             .unwrap()
                     )?);
                     Ok(())
