@@ -52,6 +52,11 @@ pub trait Runtime: Send + 'static {
     fn spawn<F>(fut: F) -> Self::JoinHandle
     where
         F: Future<Output = ()> + Send + 'static;
+
+    /// Spawn a function onto this runtime's blocking event loop
+    fn spawn_blocking<F>(f: F) -> Self::JoinHandle
+    where
+        F: FnOnce() + Send + 'static;
 }
 
 /// Extension trait for async/await runtimes that support spawning local tasks
@@ -91,7 +96,7 @@ where
     R: ContextExt,
 {
     if let Some(locals) = R::get_task_locals() {
-        Ok(locals.event_loop.into_bound(py))
+        Ok(locals.0.event_loop.clone_ref(py).into_bound(py))
     } else {
         get_running_loop(py)
     }
@@ -161,6 +166,10 @@ where
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
@@ -210,7 +219,7 @@ where
         },
     )?;
 
-    event_loop.call_method1("run_until_complete", (coro,))?;
+    event_loop.call_method1(pyo3::intern!(py, "run_until_complete"), (coro,))?;
 
     let result = result_rx.lock().unwrap().take().unwrap();
     Ok(result)
@@ -265,6 +274,10 @@ where
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
@@ -303,7 +316,7 @@ where
     F: Future<Output = PyResult<T>> + Send + 'static,
     T: Send + Sync + 'static,
 {
-    let event_loop = asyncio(py)?.call_method0("new_event_loop")?;
+    let event_loop = asyncio(py)?.call_method0(pyo3::intern!(py, "new_event_loop"))?;
 
     let result = run_until_complete::<R, F, T>(&event_loop, fut);
 
@@ -313,7 +326,10 @@ where
 }
 
 fn cancelled(future: &Bound<PyAny>) -> PyResult<bool> {
-    future.getattr("cancelled")?.call0()?.is_truthy()
+    future
+        .getattr(pyo3::intern!(future.py(), "cancelled"))?
+        .call0()?
+        .is_truthy()
 }
 
 #[pyclass]
@@ -346,8 +362,14 @@ fn set_result(
     let none = py.None().into_bound(py);
 
     let (complete, val) = match result {
-        Ok(val) => (future.getattr("set_result")?, val.into_pyobject(py)?),
-        Err(err) => (future.getattr("set_exception")?, err.into_bound_py_any(py)?),
+        Ok(val) => (
+            future.getattr(pyo3::intern!(py, "set_result"))?,
+            val.into_pyobject(py)?,
+        ),
+        Err(err) => (
+            future.getattr(pyo3::intern!(py, "set_exception"))?,
+            err.into_bound_py_any(py)?,
+        ),
     };
     call_soon_threadsafe(event_loop, &none, (CheckedCompletor, future, complete, val))?;
 
@@ -415,6 +437,10 @@ fn set_result(
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
@@ -540,6 +566,10 @@ where
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
@@ -581,13 +611,13 @@ pub fn future_into_py_with_locals<R, F, T>(
 where
     R: Runtime + ContextExt,
     F: Future<Output = PyResult<T>> + Send + 'static,
-    T: for<'py> IntoPyObject<'py>,
+    T: for<'py> IntoPyObject<'py> + Send + 'static,
 {
     let (cancel_tx, cancel_rx) = oneshot::channel();
 
-    let py_fut = create_future(locals.event_loop.bind(py).clone())?;
+    let py_fut = create_future(locals.0.event_loop.bind(py).clone())?;
     py_fut.call_method1(
-        "add_done_callback",
+        pyo3::intern!(py, "add_done_callback"),
         (PyDoneCallback {
             cancel_tx: Some(cancel_tx),
         },),
@@ -597,53 +627,59 @@ where
     let future_tx2 = future_tx1.clone_ref(py);
 
     R::spawn(async move {
-        let locals2 = Python::attach(|py| locals.clone_ref(py));
+        let locals2 = locals.clone();
 
         if let Err(e) = R::spawn(async move {
             let result = R::scope(
-                Python::attach(|py| locals2.clone_ref(py)),
+                locals2.clone(),
                 Cancellable::new_with_cancel_rx(fut, cancel_rx),
             )
             .await;
 
-            Python::attach(move |py| {
-                if cancelled(future_tx1.bind(py))
-                    .map_err(dump_err(py))
-                    .unwrap_or(false)
-                {
-                    return;
-                }
-
-                let _ = set_result(
-                    &locals2.event_loop(py),
-                    future_tx1.bind(py),
-                    result.and_then(|val| val.into_py_any(py)),
-                )
-                .map_err(dump_err(py));
-            });
-        })
-        .await
-        {
-            if e.is_panic() {
+            // We should not hold GIL inside async-std/tokio event loop,
+            // because a blocked task may prevent other tasks from progressing.
+            R::spawn_blocking(|| {
                 Python::attach(move |py| {
-                    if cancelled(future_tx2.bind(py))
+                    if cancelled(future_tx1.bind(py))
                         .map_err(dump_err(py))
                         .unwrap_or(false)
                     {
                         return;
                     }
 
-                    let panic_message = format!(
-                        "rust future panicked: {}",
-                        get_panic_message(&e.into_panic())
-                    );
                     let _ = set_result(
-                        locals.event_loop.bind(py),
-                        future_tx2.bind(py),
-                        Err(RustPanic::new_err(panic_message)),
+                        &locals2.event_loop(py),
+                        future_tx1.bind(py),
+                        result.and_then(|val| val.into_py_any(py)),
                     )
                     .map_err(dump_err(py));
                 });
+            });
+        })
+        .await
+        {
+            if e.is_panic() {
+                R::spawn_blocking(|| {
+                    Python::attach(move |py| {
+                        if cancelled(future_tx2.bind(py))
+                            .map_err(dump_err(py))
+                            .unwrap_or(false)
+                        {
+                            return;
+                        }
+
+                        let panic_message = format!(
+                            "rust future panicked: {}",
+                            get_panic_message(&e.into_panic())
+                        );
+                        let _ = set_result(
+                            locals.0.event_loop.bind(py),
+                            future_tx2.bind(py),
+                            Err(RustPanic::new_err(panic_message)),
+                        )
+                        .map_err(dump_err(py));
+                    });
+                });
             }
         }
     });
@@ -812,6 +848,10 @@ impl PyDoneCallback {
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
@@ -844,7 +884,7 @@ pub fn future_into_py<R, F, T>(py: Python, fut: F) -> PyResult<Bound<PyAny>>
 where
     R: Runtime + ContextExt,
     F: Future<Output = PyResult<T>> + Send + 'static,
-    T: for<'py> IntoPyObject<'py>,
+    T: for<'py> IntoPyObject<'py> + Send + 'static,
 {
     future_into_py_with_locals::<R, F, T>(py, get_current_locals::<R>(py)?, fut)
 }
@@ -921,6 +961,10 @@ where
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
@@ -990,9 +1034,9 @@ where
 {
     let (cancel_tx, cancel_rx) = oneshot::channel();
 
-    let py_fut = create_future(locals.event_loop.clone_ref(py).into_bound(py))?;
+    let py_fut = create_future(locals.0.event_loop.clone_ref(py).into_bound(py))?;
     py_fut.call_method1(
-        "add_done_callback",
+        pyo3::intern!(py, "add_done_callback"),
         (PyDoneCallback {
             cancel_tx: Some(cancel_tx),
         },),
@@ -1002,11 +1046,11 @@ where
     let future_tx2 = future_tx1.clone_ref(py);
 
     R::spawn_local(async move {
-        let locals2 = Python::attach(|py| locals.clone_ref(py));
+        let locals2 = locals.clone();
 
         if let Err(e) = R::spawn_local(async move {
             let result = R::scope_local(
-                Python::attach(|py| locals2.clone_ref(py)),
+                locals2.clone(),
                 Cancellable::new_with_cancel_rx(fut, cancel_rx),
             )
             .await;
@@ -1020,7 +1064,7 @@ where
                 }
 
                 let _ = set_result(
-                    locals2.event_loop.bind(py),
+                    locals2.0.event_loop.bind(py),
                     future_tx1.bind(py),
                     result.and_then(|val| val.into_py_any(py)),
                 )
@@ -1043,7 +1087,7 @@ where
                         get_panic_message(&e.into_panic())
                     );
                     let _ = set_result(
-                        locals.event_loop.bind(py),
+                        locals.0.event_loop.bind(py),
                         future_tx2.bind(py),
                         Err(RustPanic::new_err(panic_message)),
                     )
@@ -1126,6 +1170,10 @@ where
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
@@ -1240,6 +1288,10 @@ where
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
@@ -1302,7 +1354,8 @@ where
     R: Runtime,
 {
     let (tx, rx) = async_channel::bounded(1);
-    let anext: Py<PyAny> = gen.getattr("__anext__")?.into();
+    let py = gen.py();
+    let anext: Py<PyAny> = gen.getattr(pyo3::intern!(py, "__anext__"))?.into();
 
     R::spawn(async move {
         loop {
@@ -1389,6 +1442,10 @@ where
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
@@ -1506,12 +1563,7 @@ struct SenderGlue {
 #[pymethods]
 impl SenderGlue {
     pub fn send(&mut self, item: Py<PyAny>) -> PyResult<Py<PyAny>> {
-        Python::attach(|py| {
-            self.tx
-                .lock()
-                .unwrap()
-                .send(py, self.locals.clone_ref(py), item)
-        })
+        Python::attach(|py| self.tx.lock().unwrap().send(py, self.locals.clone(), item))
     }
     pub fn close(&mut self) -> PyResult<()> {
         self.tx.lock().unwrap().close()
@@ -1589,6 +1641,10 @@ async def forward(gen, sender):
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
@@ -1670,11 +1726,13 @@ where
     let (tx, rx) = mpsc::channel(10);
 
     locals.event_loop(py).call_method1(
-        "call_soon_threadsafe",
+        pyo3::intern!(py, "call_soon_threadsafe"),
         (
-            locals.event_loop(py).getattr("create_task")?,
+            locals
+                .event_loop(py)
+                .getattr(pyo3::intern!(py, "create_task"))?,
             glue.call_method1(
-                "forward",
+                pyo3::intern!(py, "forward"),
                 (
                     gen,
                     SenderGlue {
@@ -1742,6 +1800,10 @@ where
 /// #     {
 /// #         unreachable!()
 /// #     }
+/// #
+/// #     fn spawn_blocking<F>(f: F) -> Self::JoinHandle where F: FnOnce() + Send + 'static {
+/// #         unreachable!()
+/// #     }
 /// # }
 /// #
 /// # impl ContextExt for MyCustomRuntime {
