@@ -93,7 +93,7 @@
 //!     let locals = pyo3_async_runtimes::TaskLocals::with_running_loop(py)?.copy_context(py)?;
 //!
 //!     // Convert the async move { } block to a Python awaitable
-//!     pyo3_async_runtimes::tokio::future_into_py_with_locals(py, locals.clone_ref(py), async move {
+//!     pyo3_async_runtimes::tokio::future_into_py_with_locals(py, locals.clone(), async move {
 //!         let py_sleep = Python::attach(|py| {
 //!             // Sometimes we need to call other async Python functions within
 //!             // this future. In order for this to work, we need to track the
@@ -162,9 +162,9 @@
 //!
 //!     pyo3_async_runtimes::tokio::future_into_py_with_locals(
 //!         py,
-//!         locals.clone_ref(py),
+//!         locals.clone(),
 //!         // Store the current locals in task-local data
-//!         pyo3_async_runtimes::tokio::scope(locals.clone_ref(py), async move {
+//!         pyo3_async_runtimes::tokio::scope(locals.clone(), async move {
 //!             let py_sleep = Python::attach(|py| {
 //!                 pyo3_async_runtimes::into_future_with_locals(
 //!                     // Now we can get the current locals through task-local data
@@ -189,9 +189,9 @@
 //!
 //!     pyo3_async_runtimes::tokio::future_into_py_with_locals(
 //!         py,
-//!         locals.clone_ref(py),
+//!         locals.clone(),
 //!         // Store the current locals in task-local data
-//!         pyo3_async_runtimes::tokio::scope(locals.clone_ref(py), async move {
+//!         pyo3_async_runtimes::tokio::scope(locals.clone(), async move {
 //!             let py_sleep = Python::attach(|py| {
 //!                 pyo3_async_runtimes::into_future_with_locals(
 //!                     &pyo3_async_runtimes::tokio::get_current_locals(py)?,
@@ -395,6 +395,7 @@ pub mod doc_test {
 }
 
 use std::future::Future;
+use std::sync::Arc;
 
 use futures::channel::oneshot;
 use pyo3::{call::PyCallArgs, prelude::*, sync::PyOnceLock, types::PyDict};
@@ -407,31 +408,34 @@ static GET_RUNNING_LOOP: PyOnceLock<Py<PyAny>> = PyOnceLock::new();
 fn ensure_future<'p>(py: Python<'p>, awaitable: &Bound<'p, PyAny>) -> PyResult<Bound<'p, PyAny>> {
     ENSURE_FUTURE
         .get_or_try_init(py, || -> PyResult<Py<PyAny>> {
-            Ok(asyncio(py)?.getattr("ensure_future")?.into())
+            Ok(asyncio(py)?
+                .getattr(pyo3::intern!(py, "ensure_future"))?
+                .into())
         })?
         .bind(py)
         .call1((awaitable,))
 }
 
 fn create_future(event_loop: Bound<'_, PyAny>) -> PyResult<Bound<'_, PyAny>> {
-    event_loop.call_method0("create_future")
+    event_loop.call_method0(pyo3::intern!(event_loop.py(), "create_future"))
 }
 
 fn close(event_loop: Bound<PyAny>) -> PyResult<()> {
+    let py = event_loop.py();
     event_loop.call_method1(
-        "run_until_complete",
-        (event_loop.call_method0("shutdown_asyncgens")?,),
+        pyo3::intern!(py, "run_until_complete"),
+        (event_loop.call_method0(pyo3::intern!(py, "shutdown_asyncgens"))?,),
     )?;
 
     // how to do this prior to 3.9?
-    if event_loop.hasattr("shutdown_default_executor")? {
+    if event_loop.hasattr(pyo3::intern!(py, "shutdown_default_executor"))? {
         event_loop.call_method1(
-            "run_until_complete",
-            (event_loop.call_method0("shutdown_default_executor")?,),
+            pyo3::intern!(py, "run_until_complete"),
+            (event_loop.call_method0(pyo3::intern!(py, "shutdown_default_executor"))?,),
         )?;
     }
 
-    event_loop.call_method0("close")?;
+    event_loop.call_method0(pyo3::intern!(py, "close"))?;
 
     Ok(())
 }
@@ -452,7 +456,9 @@ pub fn get_running_loop(py: Python) -> PyResult<Bound<PyAny>> {
         .get_or_try_init(py, || -> PyResult<Py<PyAny>> {
             let asyncio = asyncio(py)?;
 
-            Ok(asyncio.getattr("get_running_loop")?.into())
+            Ok(asyncio
+                .getattr(pyo3::intern!(py, "get_running_loop"))?
+                .into())
         })?
         .bind(py)
         .call0()
@@ -465,25 +471,29 @@ fn contextvars(py: Python<'_>) -> PyResult<&Bound<'_, PyAny>> {
 }
 
 fn copy_context(py: Python) -> PyResult<Bound<PyAny>> {
-    contextvars(py)?.call_method0("copy_context")
+    contextvars(py)?.call_method0(pyo3::intern!(py, "copy_context"))
 }
 
-/// Task-local data to store for Python conversions.
+/// Task-local inner structure.
 #[derive(Debug)]
-pub struct TaskLocals {
+struct TaskLocalsInner {
     /// Track the event loop of the Python task
     event_loop: Py<PyAny>,
     /// Track the contextvars of the Python task
     context: Py<PyAny>,
 }
 
+/// Task-local data to store for Python conversions.
+#[derive(Debug)]
+pub struct TaskLocals(Arc<TaskLocalsInner>);
+
 impl TaskLocals {
     /// At a minimum, TaskLocals must store the event loop.
     pub fn new(event_loop: Bound<PyAny>) -> Self {
-        Self {
+        Self(Arc::new(TaskLocalsInner {
             context: event_loop.py().None(),
             event_loop: event_loop.into(),
-        }
+        }))
     }
 
     /// Construct TaskLocals with the event loop returned by `get_running_loop`
@@ -493,10 +503,10 @@ impl TaskLocals {
 
     /// Manually provide the contextvars for the current task.
     pub fn with_context(self, context: Bound<PyAny>) -> Self {
-        Self {
+        Self(Arc::new(TaskLocalsInner {
+            event_loop: self.0.event_loop.clone_ref(context.py()),
             context: context.into(),
-            ..self
-        }
+        }))
     }
 
     /// Capture the current task's contextvars
@@ -506,21 +516,26 @@ impl TaskLocals {
 
     /// Get a reference to the event loop
     pub fn event_loop<'p>(&self, py: Python<'p>) -> Bound<'p, PyAny> {
-        self.event_loop.clone_ref(py).into_bound(py)
+        self.0.event_loop.clone_ref(py).into_bound(py)
     }
 
     /// Get a reference to the python context
     pub fn context<'p>(&self, py: Python<'p>) -> Bound<'p, PyAny> {
-        self.context.clone_ref(py).into_bound(py)
+        self.0.context.clone_ref(py).into_bound(py)
     }
 
-    /// Create a clone of the TaskLocals by incrementing the reference counters of the event loop and
-    /// contextvars.
-    pub fn clone_ref(&self, py: Python<'_>) -> Self {
-        Self {
-            event_loop: self.event_loop.clone_ref(py),
-            context: self.context.clone_ref(py),
-        }
+    /// Create a clone of the TaskLocals. No longer uses the runtime, use `clone` instead.
+    #[deprecated(note = "please use `clone` instead")]
+    pub fn clone_ref(&self, _py: Python<'_>) -> Self {
+        self.clone()
+    }
+}
+
+impl Clone for TaskLocals {
+    /// Create a clone of the TaskLocals by incrementing the reference counter of the inner
+    /// structure.
+    fn clone(&self) -> Self {
+        Self(self.0.clone())
     }
 }
 
@@ -533,8 +548,9 @@ struct PyTaskCompleter {
 impl PyTaskCompleter {
     #[pyo3(signature = (task))]
     pub fn __call__(&mut self, task: &Bound<PyAny>) -> PyResult<()> {
-        debug_assert!(task.call_method0("done")?.extract()?);
-        let result = match task.call_method0("result") {
+        let py = task.py();
+        debug_assert!(task.call_method0(pyo3::intern!(py, "done"))?.extract()?);
+        let result = match task.call_method0(pyo3::intern!(py, "result")) {
             Ok(val) => Ok(val.into()),
             Err(e) => Err(e),
         };
@@ -565,7 +581,7 @@ impl PyEnsureFuture {
         Python::attach(|py| {
             let task = ensure_future(py, self.awaitable.bind(py))?;
             let on_complete = PyTaskCompleter { tx: self.tx.take() };
-            task.call_method1("add_done_callback", (on_complete,))?;
+            task.call_method1(pyo3::intern!(py, "add_done_callback"), (on_complete,))?;
 
             Ok(())
         })
@@ -580,9 +596,13 @@ fn call_soon_threadsafe<'py>(
     let py = event_loop.py();
 
     let kwargs = PyDict::new(py);
-    kwargs.set_item("context", context)?;
+    kwargs.set_item(pyo3::intern!(py, "context"), context)?;
 
-    event_loop.call_method("call_soon_threadsafe", args, Some(&kwargs))?;
+    event_loop.call_method(
+        pyo3::intern!(py, "call_soon_threadsafe"),
+        args,
+        Some(&kwargs),
+    )?;
     Ok(())
 }
 
@@ -659,7 +679,7 @@ pub fn into_future_with_locals(
             Ok(item) => item,
             Err(_) => Python::attach(|py| {
                 Err(PyErr::from_value(
-                    asyncio(py)?.call_method0("CancelledError")?,
+                    asyncio(py)?.call_method0(pyo3::intern!(py, "CancelledError"))?,
                 ))
             }),
         }
